'*******************************************************************************
'*                           TListView.bi                                      *
'*             This file is a part of sGUI libary for FreeBasic                *
'*             Copyright 2010-2013 Nastasa Vasile Eodor                       *
'*             http://sgui.rqwork.evonet.ro                                    *
'*             mail: nastase.eodor@gmail.com                                   *
'*                           version 2.0                                       *
'*******************************************************************************

#include once "control.bi"
#include once "graphics.bi"
#include once "menus.bi"

#include once "sysutils.bi"

#ifndef TListViewClass
#define TListViewClass(__Ptr__) *cast(TListView Ptr,__Ptr__)
#endif

#ifndef GetListView
#define GetListView(__HWND__) *cast(TListView Ptr,GetWindowLong(__HWND__,GetClassLong(__HWND__,gcl_cbwndextra)-4))
#endif

type PListView as TListView ptr

type TListViewItem
    private:
    FCaption       as string
    FImageIndex    as integer
    public:
    ListView       as PListView
    Parent         as TListViewItem ptr
    Count          as integer
    Subitems       as TListViewItem ptr ptr
    Data           as any ptr
    Tag            as any ptr
    declare constructor
    declare destructor
    declare operator cast as any ptr
    declare operator cast as string
    declare property Caption as string
    declare property Caption(value as string)
    declare property ImageIndex as integer
    declare property ImageIndex(value as integer)
    declare property Subitem(index as integer) as TListViewItem ptr 
    declare property Subitem(index as integer,value as TListViewItem ptr)
    declare function IndexOf(value as TListViewItem ptr) as integer
end type

type TListViewColumn
    private:
    FWidth     as integer
    FAlignment as integer
    FCaption   as string
    public:
    ListView   as PListView
    Tag        as any ptr
    declare constructor
    declare destructor
    declare operator cast as any ptr
    declare operator cast as string
    declare property Width as integer
    declare property Width(value as integer)
    declare property Caption as string
    declare property Caption(value as string)
    declare property Alignment as integer
    declare property Alignment(value as integer)
end type

Type TListView
    private:
    AControl       as TControl = TControl("TListView","SysListView32")
    FStyle         as integer
    FViewStyle     as integer
    FButton        as integer
    FLabelWrap     as integer
    FReadOnly      as integer
    FAutoarrange   as integer
    FShowColumns   as integer
    FItemIndex     as integer
    FCheckBoxes    as integer
    FColCount      as integer
    FColumns       as TListViewColumn ptr ptr
    FCount         as integer
    FItems         as TListViewItem ptr ptr
    declare sub DispatchMessage(Byref Message as TMessage)
    declare sub ProcessMessage(Byref Message as TMessage)
    declare sub MsgDefault(Byref Message as TMessage)
    declare static function WindowProc(fWindow as hWnd,msg as UInt,wparam as wParam,lparam as lParam) as LResult
    public:
    SmallImages    as TImageList
    LargeImages    as TImageList
    StateImages    as TImageList
    Font           as TFont
    PopupMenu      as TPopupMenu
    Cursor         as TCursor
    Tag            as any ptr
    declare sub AddColumn(ACaption as string="",AWidth as integer=50,AAlignment as integer=0)
    declare sub InsertColumn(index as integer,ACaption as string="",AWidth as integer=50,AAlignment as integer=0)
    declare sub DeleteColumn(index as integer)
    declare sub AddItem(ACaption as string="")
    declare sub InsertItem(index as integer,ACaption as string="")
    declare sub AddSubitem(itemindex as integer,ACaption as string="")
    declare sub InsertSubitem(itemindex as integer,index as integer,ACaption as string="")
    declare sub DeleteItem(index as integer)
    declare function IndexOf(value as TListViewItem ptr) as integer
    declare sub Clear
    declare sub ClearAll
    declare constructor
    declare destructor
    declare operator cast as any ptr
    declare operator cast as TControl ptr
    declare operator cast as string
    declare static Sub RegisterClass(ClassName as string = "TListView",ClassAncestor as string = "SysListView32")
    declare property Style as Integer
    declare property Style(value as Integer)
    declare property ViewStyle as Integer
    declare property ViewStyle(value as Integer)
    declare property ShowColumns as Integer
    declare property ShowColumns(value as Integer)
    declare property Button as Integer
    declare property Button(value as Integer)
    declare property Autoarrange as Integer
    declare property Autoarrange(value as Integer)
    declare property LabelWrap as Integer
    declare property LabelWrap(value as Integer)
    declare property ReadOnly as Integer
    declare property ReadOnly(value as Integer)
    declare property ColCount as Integer
    declare property ColCount(value as Integer)
    declare property Count as Integer
    declare property Count(value as Integer)
    declare property Item(index as integer) as TListViewItem ptr 
    declare property Item(index as integer,value as TListViewItem ptr)
    declare property Align as Integer
    declare property Align(value as Integer)
    declare property ClientHeight as Integer
    declare property ClientHeight(value as Integer)
    declare property ClientWidth as Integer
    declare property ClientWidth(value as Integer)
    declare property Enabled as Integer
    declare property Enabled(value as Integer)
    declare property Handle as hwnd
    declare property Handle(value as hwnd)
    declare property Height as Integer
    declare property Height(value as Integer)
    declare property Hint as String
    declare property Hint(value as String)
    declare property Left as Integer
    declare property Left(value as Integer)
    declare property Parent as TControl ptr
    declare property Parent(value as TControl ptr)
    declare property ShowHint as Integer
    declare property ShowHint(value as Integer)
    declare property TabOrder as Integer
    declare property TabOrder(value as Integer)
    declare property TabStop as Integer
    declare property TabStop(value as Integer)
    declare property Top as Integer
    declare property Top(value as Integer)
    declare property Visible as Integer
    declare property Visible(value as Integer)
    declare property Width as Integer
    declare property Width(value as Integer)
    declare sub BringToFront
    declare sub ClientToScreen(Byref P as Point)
    declare sub Invalidate
    declare sub Repaint
    declare sub ScreenToClient(Byref P as Point)
    declare sub SendToBack
    declare sub SetFocus
    declare sub Update
    onClick      as sub(byref Sender as TListView)
    onDblClick   as sub(byref Sender as TListView)
    onSelChanged as sub(byref Sender as TListView)
    onDrawItem   as sub(byref Sender as TListView,Index as integer,R as Rect,State as integer)
end Type
