'*******************************************************************************
'*                             TRichEdit.bi                                    *
'*             This file is a part of sGUI libary for FreeBasic                *
'*             Copyright 2010-2013 Nastasa Vasile Eodor                       *
'*             http://sgui.rqwork.evonet.ro                                    *
'*             mail: nastase.eodor@gmail.com                                   *
'*                             version 2.0                                     *
'*******************************************************************************

#include once "graphics.bi"
#include once "menus.bi"
#include once "control.bi"

#ifdef em_getseltext
#undef em_getseltext
#endif

#include once "win/richedit.bi"

#ifndef TRichEditClass 
#define TRichEditClass(__Ptr__) *cast(TRichEdit Ptr,__Ptr__)
#endif


enum TNumberingStyle 
    nsNone,nsBullet
end enum

enum TAlignment
    taLeft,taCenter,taRight
end enum

type PRichEdit as TRichEdit ptr

type TParaAttributes 
    private:
    FAlignment   as TAlignment
    FFirstIndent as integer
    FLeftIndent  as integer
    FNumbering   as TNumberingStyle
    FRightIndent as integer
    FTab         as integer
    FTabCount    as integer
    declare sub GetAttributes(byref P as PARAFORMAT)
    declare sub SetAttributes(byref P as PARAFORMAT)
    public:
    RichEdit     as PRichEdit
    Paragraph    as PARAFORMAT
    declare property Alignment as TAlignment
    declare property Alignment(value as TAlignment)
    declare property FirstIndent as integer
    declare property FirstIndent(value as integer)
    declare property LeftIndent as integer
    declare property LeftIndent(value as integer)
    declare property Numbering as TNumberingStyle
    declare property Numbering(value as TNumberingStyle)
    declare property RightIndent as integer
    declare property RightIndent(value as integer)
    declare property Tab(Index as Byte) as integer
    declare property Tab(Index as Byte,value as integer)
    declare property TabCount as integer
    declare property TabCount(value as integer)
    declare operator cast as any ptr
    declare operator cast as string
    declare constructor
    declare destructor
end type

type TTextAttributes
    private:
    FName      as string 
    FColor     as integer
    FSize      as integer
    FCharSet   as integer
    FPitch     as integer
    FBold      as integer
    FItalic    as integer
    FUnderLine as integer
    FStrikeOut as integer
    declare sub GetAttributes(byref CF as CHARFORMAT)
    declare sub SetAttributes(byref CF as CHARFORMAT)
    public:
    RichEdit   as PRichEdit
    Format     as CHARFORMAT
    Style      as integer
    declare property Color as integer
    declare property Color(value as integer)
    declare property Name as string
    declare property Name(value as string)
    declare property Size as integer
    declare property Size(value as integer)
    declare property CharSet as integer
    declare property CharSet(value as integer)
    declare property Bold as integer
    declare property Bold(value as integer)
    declare property Italic as integer
    declare property Italic(value as integer)
    declare property Underline as integer
    declare property Underline(value as integer)
    declare property StrikeOut as integer
    declare property StrikeOut(value as integer)
    declare operator cast as any ptr
    declare operator cast as string
    declare operator cast as TFont
    declare operator let(value as TFont)
    declare constructor
    declare destructor
end type

type TRichEditStrings
    private:
    FText    as string
    FCount   as integer
    FLines   as zstring ptr ptr
    public:
    RichEdit as PRichEdit
    declare property Count as integer
    declare property Count(value as integer)
    declare property Line(Index as integer) as string
    declare property Line(Index as integer,value as string)
    declare property Text as string
    declare property Text(value as string)
    declare property Item(index as integer) as string
    declare property Item(index as integer,value as string)
    declare sub Add(value as string)
    declare sub Insert(Index as integer,value as string)
    declare sub Remove(Index as integer)
    declare sub Exchange(Index1 as integer,Index2 as integer)
    declare function IndexOf(value as string) as integer
    declare sub Clear
    declare sub LoadFromFile(File as string)
    declare sub SaveToFile(File as string)
    declare operator cast as any ptr
    declare operator cast as string
    declare constructor
    declare destructor
end type

type TRichEdit
    private:
    AControl           as TControl=TControl("TRichEdit","RichEdit20A")
    FSelStart          as integer
    FSelLength         as integer
    FSelText           as string
    FCharCase          as integer
    FMasked            as integer
    FMaskChar          as Byte
    FBorderStyle       as integer
    FReadOnly          as integer
    FCtl3D             as integer
    FHideSelection     as integer
    FPlainText         as integer
    FMaxLength         as integer 
    FModified          as integer
    FCaretPos          as Point
    FHideScrollBars    as integer
    declare static function WindowProc(FWindow as hwnd,Msg as uint,wparam as wparam,lparam as lparam) as lresult
    declare sub ProcessMessage(byref Message as TMessage)
    declare sub DispatchMessage(byref message as TMessage)
    declare sub MsgDefault(byref message as TMessage)
    public:
    Paragraph          as TParaAttributes
    SelAttributes      as TTextAttributes 
    Font               as TFont  
    Lines              as TRichEditStrings
    PopupMenu          as TPopupMenu
    Tag                as any ptr
    declare property Text as string
    declare property Text(value as string)
    declare property BorderStyle as integer
    declare property BorderStyle(value as integer)
    declare property ReadOnly as integer
    declare property ReadOnly(value as integer)
    declare property HideSelection as integer
    declare property HideSelection(value as integer)
    declare property PlainText as integer
    declare property PlainText(value as integer)
    declare property MaxLength as integer
    declare property MaxLength(value as integer)
    declare property Modified as integer
    declare property Modified(value as integer)   
    declare property CharCase as integer
    declare property CharCase(value as integer)
    declare property SelStart as integer
    declare property SelStart(value as integer)
    declare property SelLength as integer
    declare property SelLength(value as integer)
    declare property SelText as string
    declare property SelText(value as string)
    declare property CaretPos as Point
    declare property CaretPos(value as Point)
    declare property HideScrollBars as integer
    declare property HideScrollBars(value as integer)
    declare property Align as integer
    declare property Align(value as integer)
    declare property Left as integer
    declare property Left(value as integer)
    declare property Top as integer
    declare property Top(value as integer)
    declare property Width as integer
    declare property Width(value as integer)
    declare property Height as integer
    declare property Height(value as integer)
    declare property ClientWidth as integer
    declare property ClientHeight as integer
    declare property Color as integer
    declare property Color(value as integer)
    declare property Handle as HWND
    declare property Handle(value as HWND)
    declare property Parent as TControl ptr
    declare property Parent(value as TControl ptr)
    declare property Visible as integer
    declare property Visible(value as integer)
    declare property Enabled as integer
    declare property Enabled(value as integer)
    declare operator cast as TControl ptr
    declare operator cast as string
    declare operator cast as any ptr
    declare static sub RegisterClass(byref ClassName as string="TRichEdit",byref ClassAncestor as string="RichEdit20A")
    declare function Perform(Msg as UINT,wParam as WPARAM,lParam as LPARAM) as LRESULT
    declare sub Recreate
    declare sub Free
    declare sub Invalidate
    declare sub Update
    declare sub Repaint
    declare sub SetFocus
    declare sub BringToFront
    declare sub SendToBack
    declare sub ClientToScreen(byref P as Point)
    declare sub ScreenToClient(byref P as Point)
    declare sub Clear
    declare sub ClearUndo
    declare function CanUndo as integer
    declare sub Undo
    declare sub PasteFromClipboard
    declare sub CopyToClipboard
    declare sub CutToClipboard
    declare sub SelectAll
    declare constructor
    declare destructor
    onChange         as sub(byref Sender as TRichEdit)
    onSelChange      as sub(byref Sender as TRichEdit)
    onProtectChange  as sub(byref Sender as TRichEdit,SelStart as integer,SelEnd as integer,byref AllowChange as integer)
    onResize         as sub(byref Sender as TRichEdit,R as Rect)
    onSaveClipBoard  as sub(byref Sender as TRichEdit,Objects as integer,Chars as integer,byref AllowSave as integer)
    onKeyPress       as sub(byref Sender as TRichEdit,Key as Byte)
    onKeyDown        as sub(byref Sender as TRichEdit,Key as integer,Shift as integer)
    onKeyUp          as sub(byref Sender as TRichEdit,Key as integer,Shift as integer)
end type
