'*******************************************************************************
'*                           TTreeView.bi                                      *
'*             This file is a part of sGUI libary for FreeBasic                *
'*             Copyright 2010-2013 Nastasa Vasile Eodor                       *
'*             http://sgui.rqwork.evonet.ro                                    *
'*             mail: nastase.eodor@gmail.com                                   *
'*                           version 2.0                                       *
'*******************************************************************************

#include once "control.bi"
#include once "graphics.bi"
#include once "menus.bi"

#ifndef TTreeViewClass
#define TTreeViewClass(__Ptr__) *cast(TTreeView Ptr,__Ptr__)
#endif

#ifndef GetTreeView
#define GetTreeView(__HWND__) *cast(TTreeView Ptr,GetWindowLong(__HWND__,GetClassLong(__HWND__,gcl_cbwndextra)-4))
#endif

type PTreeView as TTreeView ptr

type TTreeItem
    private:
    FText          as string
    FBold          as integer 
    FImageIndex    as integer
    FStateIndex    as integer
    FSelectedIndex as integer
    public:
    TreeView       as PTreeView
    Handle         as htreeitem
    Parent         as TTreeItem ptr
    Data           as any ptr
    Tag            as any ptr
    declare constructor
    declare destructor
    declare operator cast as any ptr
    declare operator cast as string
    declare property Text as string
    declare property Text(value as string)
    declare property Bold as integer
    declare property Bold(value as integer)
    declare property ImageIndex as integer
    declare property ImageIndex(value as integer)
    declare property StateIndex as integer
    declare property StateIndex(value as integer)
    declare property SelectedIndex as integer
    declare property SelectedIndex(value as integer)
end type

Type TTreeView
    private:
    AControl       as TControl = TControl("TTreeView","SysTreeView32")
    FHasButtons    as integer
    FHaslines      as integer
    FLinesRoot     as integer
    FShowSelection as integer
    FHottrack      as integer
    FSelected      as TTreeItem ptr
    FSelIndex      as integer
    FReadOnly      as integer
    FRowSelect     as integer
    FTopItem       as TTreeItem ptr
    FTopIndex      as integer
    FToolTips      as integer
    FCount         as integer
    FItems         as TTreeItem ptr ptr
    declare sub DispatchMessage(Byref Message as TMessage)
    declare sub ProcessMessage(Byref Message as TMessage)
    declare sub MsgDefault(Byref Message as TMessage)
    declare static function WindowProc(fWindow as hWnd,msg as UInt,wparam as wParam,lparam as lParam) as LResult
    public:
    Images         as TImageList
    StateImages    as TImageList
    Font           as TFont
    PopupMenu      as TPopupMenu
    Cursor         as TCursor
    Tag            as any ptr
    declare function Add(AText as string="",AData as any ptr=0) as TTreeItem ptr
    declare function AddChild(AParent as TTreeItem ptr,AText as string="",AData as any ptr=0) as TTreeItem ptr
    declare function Insert(index as integer,AText as string="",AData as any ptr=0)as TTreeItem ptr
    declare function InsertChild(AParent as TTreeItem ptr,index as integer,AText as string="",AData as any ptr=0)as TTreeItem ptr
    declare sub Remove(value as TTreeItem ptr)
    declare function IndexOf(value as TTreeItem ptr) as integer
    declare function FindItem(value as htreeitem) as TTreeItem ptr
    declare sub Clear
    declare sub FullExpand
    declare sub FullCollapse
    declare Constructor
    declare Destructor
    declare Operator cast as any ptr
    declare Operator cast as TControl ptr
    declare Operator cast as string
    declare static sub RegisterClass(ClassName as string = "TTreeView",ClassAncestor as string = "SysTreeView32")
    declare property Item(index as Integer) as TTreeItem ptr
    declare property Item(index as Integer,value as TTreeItem ptr)
    declare property ItemData(index as Integer) as any ptr
    declare property ItemData(index as Integer,value as any ptr)
    declare property ShowSelection as Integer
    declare property ShowSelection(value as Integer)
    declare property HasButtons as Integer
    declare property HasButtons(value as Integer)
    declare property HasLines as Integer
    declare property HasLines(value as Integer)
    declare property LinesRoot as Integer
    declare property LinesRoot(value as Integer)
    declare property HotTrack as Integer
    declare property HotTrack(value as Integer)
    declare property ReadOnly as Integer
    declare property ReadOnly(value as Integer)
    declare property RowSelect as Integer
    declare property RowSelect(value as Integer)
    declare property ToolTips as Integer
    declare property ToolTips(value as Integer)
    declare property Selected as TTreeItem ptr
    declare property Selected(value as TTreeItem ptr)
    declare property SelIndex as integer
    declare property SelIndex(value as integer)
    declare property TopItem as TTreeItem ptr
    declare property TopItem(value as TTreeItem ptr)
    declare property TopIndex as integer
    declare property TopIndex(value as integer)
    declare property Count as Integer
    declare property Count(value as Integer)
    declare property Align as Integer
    declare property Align(value as Integer)
    declare property ClientHeight as Integer
    declare property ClientHeight(value as Integer)
    declare property ClientWidth as Integer
    declare property ClientWidth(value as Integer)
    declare property Color as Integer
    declare property Color(value as Integer)
    declare property Enabled as Integer
    declare property Enabled(value as Integer)
    declare property Handle as hwnd
    declare property Handle(value as hwnd)
    declare property Height as Integer
    declare property Height(value as Integer)
    declare property Hint as String
    declare property Hint(value as String)
    declare property Left as Integer
    declare property Left(value as Integer)
    declare property Parent as TControl ptr
    declare property Parent(value as TControl ptr)
    declare property ShowHint as Integer
    declare property ShowHint(value as Integer)
    declare property TabOrder as Integer
    declare property TabOrder(value as Integer)
    declare property TabStop as Integer
    declare property TabStop(value as Integer)
    declare property Top as Integer
    declare property Top(value as Integer)
    declare property Visible as Integer
    declare property Visible(value as Integer)
    declare property Width as Integer
    declare property Width(value as Integer)
    declare sub BringToFront
    declare sub ClientToScreen(Byref P as Point)
    declare sub Invalidate
    declare sub Repaint
    declare sub ScreenToClient(Byref P as Point)
    declare sub SendToBack
    declare sub SetFocus
    declare sub Update
    onClick            as sub(byref Sender as TTreeView)
    onDblClick         as sub(byref Sender as TTreeView)
    onSelectedChanged  as sub(byref Sender as TTreeView,ChangeBy as integer)
    onSelectedChanging as sub(byref Sender as TTreeView,ChangeBy as integer,byref AllowChange as integer)
    onDeleteItem       as sub(byref Sender as TTreeView,Node as TTreeItem ptr)
    onExpandItem       as sub(byref Sender as TTreeView,Node as TTreeItem ptr)
    onCollapseItem     as sub(byref Sender as TTreeView,Node as TTreeItem ptr) 
    onExpandingItem    as sub(byref Sender as TTreeView,Node as TTreeItem ptr,byref AllowExpand as integer)
    onCollapsingItem   as sub(byref Sender as TTreeView,Node as TTreeItem ptr,byref AllowExpand as integer) 
end Type
